from curvenote.latex.TaggedContentCollection import TaggedContentCollection


def test_init():
    c = TaggedContentCollection()
    assert list(c.keys()) == []


def test_add():
    c = TaggedContentCollection()

    c.add("a", "b")
    assert c["a"] == "b\n"

    c.add("a", "c")
    assert c["a"] == "b\n\nc\n"


def test_merge():
    c1 = TaggedContentCollection({"a": "b\n", "b": "c\n"})
    c2 = TaggedContentCollection({"a": "d\n", "c": "e\n"})

    c1.merge(c2)

    assert c1["a"] == "b\n\nd\n"
    assert c1["b"] == "c\n"
    assert c1["c"] == "e\n"



from typing import Dict


class TaggedContentCollection(dict):
    """
    A Container for tagged content that supports the appending we need
    """

    def add(self, tag: str, content: str):
        """
        Add content to the collection
        """
        if tag not in self:
            self[tag] = f"{content}\n"
            return
        self[tag] += f"\n{content}\n"

    def merge(self, other: "TaggedContentCollection"):
        """
        Merge this collection with another
        """
        for tag, content in other.items():
            if tag not in self:
                self[tag] = f"{content}"
            else:
                self[tag] += f"\n{content}"



import logging
import os
import sys

import typer
from curvenote_template import LatexBuilder

from curvenote.models import BlockFormat
from curvenote.utils import decode_oxa_link, decode_url

from ..client import Session
from ..latex import LatexProject, OnlineTemplateLoader

app = typer.Typer()

logger = logging.getLogger()


def url_set(ctx: typer.Context, value: str):
    if ctx.resilient_parsing:
        return
    if value:
        typer.echo("URL provided project, article and version options will be ignored")
    return value


def pull_as_latex(
    target: str = typer.Argument(
        ...,
        help=(
            "Local folder in which to construct the Latex assets. If TARGET exists it"
            "and all files will be removed and a new empty folder structure created"
        ),
    ),
    token: str = typer.Argument(
        ..., envvar="CURVENOTE_TOKEN", help="API token generated from curvenote.com"
    ),
    url: str = typer.Option(
        None,
        help=("A valid url (or oxa link) to the existing Article"),
        is_eager=True,
        callback=url_set,
    ),
    vanilla: bool = typer.Option(
        None,
        help=(
            "When set, will request vanilla LaTeX without Curvenote custom commands and environments"
        ),
    ),
):
    logger.setLevel(logging.INFO)
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.INFO)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    try:
        session = Session(token)
    except Exception as err:
        typer.echo("Could not start an API session - check your token")
        raise typer.Exit(code=1)

    if url:
        typer.echo(f"Accessing block {url}")
        version_id, pathspec = None, None
        try:
            version_id = decode_oxa_link(url)
        except ValueError:
            pathspec = decode_url(url)
            typer.echo("Decoded {pathspec}")

        try:
            block = session.get_block(
                version_id.project if version_id is not None else pathspec.project,
                version_id.block if version_id is not None else pathspec.block,
            )
            typer.echo(f"Found & retreived block: {block.name}")
        except ValueError as err:
            typer.echo(err)
            typer.echo("Could not find block or you do not have access")
            raise typer.Exit(code=1) from err

        try:
            latex_project = LatexProject.build_single_article_by_url(
                target,
                session,
                url,
                BlockFormat.tex if vanilla else BlockFormat.tex_curvenote,
                [],
            )
            latex_project.write()
        except ValueError as err:
            typer.echo(err)
            raise typer.Exit(code=1)

        typer.echo("Exiting...")
        return typer.Exit(code=0)



from unittest.mock import MagicMock, Mock, patch

import pytest

from curvenote.latex import LatexArticle


@patch.object(LatexArticle, "authors")
def test_author_names_lookup(mock_authors):
    a1 = Mock()
    a1.plain = None
    a1.user = "1"
    a2 = Mock()
    a2.plain = "Jane Doe"
    a2.user = None
    mock_authors.__get__ = Mock(return_value=[a1, a2])
    article = LatexArticle(None, "", "")
    article._block = MagicMock()
    u1 = Mock()
    u1.id = "1"
    u1.display_name = "User One"
    u2 = Mock()
    u2.id = "2"
    u2.display_name = "User Two"
    article._users = [u1, u2]
    authors = article.authors
    assert len(authors) == 2

    author_names = article.author_names
    assert len(author_names) == 2
    assert author_names[0] == "User One"
    assert author_names[1] == "Jane Doe"


def test_oxalink():
    a = LatexArticle(Mock(), "", "")

    a._version = Mock()

    a._version.id = Mock()
    a._version.id.project = "qwerty"
    a._version.id.block = "asdfgh"
    a._version.id.version = 3

    assert a.oxalink("https://cn.com") == "https://cn.com/oxa:qwerty/asdfgh.3"


@pytest.mark.parametrize(
    "tags1, tags2, content, keys, cases",
    [
        ([], [], "block1\n\nblock2\n", [], []),
        (["random"], [], "block1\n\nblock2\n", [], []),
        (["no-export"], [], "block2\n", [], []),
        (["abstract"], [], "block2\n", ["abstract"], [("abstract", "block1\n")]),
        (
            ["abstract"],
            ["abstract"],
            "",
            ["abstract"],
            [("abstract", "block1\n\nblock2\n")],
        ),
        (["abstract"], ["no-export"], "", ["abstract"], [("abstract", "block1\n")]),
    ],
)
def test_dump_article(tags1, tags2, content, keys, cases):
    allowed_tags = set(["abstract", "another"])

    article = LatexArticle(Mock(), "", "")

    block1 = Mock()
    block1.tags = tags1
    block1.content = "block1"

    block2 = Mock()
    block2.tags = tags2
    block2.content = "block2"

    article._latex_block_versions = [block1, block2]

    content, tagged_content = article.dump(allowed_tags)

    assert content == content
    assert list(tagged_content.keys()) == keys
    for tag, case in cases:
        assert tagged_content[tag]
        assert tagged_content[tag] == case



import logging
import os
from typing import Any, Dict, List, NamedTuple, Optional, Set, Tuple, Union

import yaml
from curvenote_template import DocModel

from curvenote.latex.TaggedContentCollection import TaggedContentCollection
from curvenote.latex.utils.decorators import log_and_raise_errors
from curvenote.models import BlockFormat, Project

from ..client import Session
from ..utils import decode_oxa_link, decode_url
from .LatexArticle import LatexArticle
from .utils import LocalMarker, escape_latex

logger = logging.getLogger()


class ProjectItem(NamedTuple):
    path: str
    filename: str
    article: LatexArticle


class LatexProject:
    """
    Responsible for project wide fetching and reconcilliation of content as a LaTeX project
    """

    def __init__(
        self,
        session: Session,
        target_folder: str,
        user_options: Dict[str, Any],
    ):
        self._session: Session = session
        self._user_options: Dict[str, Any] = user_options

        self._target_folder: str = os.path.abspath(target_folder)
        self._assets_folder: str = os.path.join(self._target_folder, "assets")
        self._images_folder: str = os.path.join(self._target_folder, "assets", "images")

        self.create_folders()

        self._project_items: List[ProjectItem] = []
        self._reference_list: List[LocalMarker] = []
        self._figure_list: List[LocalMarker] = []
        self._tagged_content: TaggedContentCollection = TaggedContentCollection()

    @classmethod
    def build_single_article_by_name(
        cls,
        target_folder: str,
        session: Session,
        project_id_or_obj: Union[str, Project],
        article_id: str,
        version: int,
        tex_format: BlockFormat,
        tagged_blocks: Set[str] = set(),
        user_options: Dict[str, Any] = {},
    ):
        """
        Factory Method

        Creates a LatexProject, performs fetch and reconcillation and then returns
        that LatexProject object

        @param cls: The LatexProject class
        @param target_folder: The folder where the project will be created
        @param session: The current session
        @param project_id_or_obj: The ID of the project or the Project object
        @param article_id: The ID of the article
        @param version: The version of the article
        @param tex_format: The format of the article
        @param tagged_blocks: The names of tagged blocks to pull from the content flow
        @param user_options: a dict of data that will correspond to user options available in the specified template
        @returns: A LatexProject object
        """
        latex_project = cls(session, target_folder, user_options)
        latex_project.add_article(project_id_or_obj, article_id, version, tex_format)
        latex_project.reconcile()
        return latex_project

    @classmethod
    def build_single_article_by_url(
        cls,
        target_folder: str,
        session: Session,
        url: str,
        tex_format: BlockFormat,
        tagged_blocks: Set[str] = set(),
        user_options: Dict[str, Any] = {},
    ):
        """
        Factory Method
        Creates a LatexProject, performs fetch and reconcilillation and then returns
        that LatexProject object

        @param cls: The LatexProject class
        @param target_folder: The folder where the project will be created
        @param session: The current session
        @param url: The URL of the article
        @param tex_format: The format of the article
        @param tagged_blocks: The names of tagged blocks to pull from the content flow
        @param user_options: a dict of data that will correspond to user options available in the specified template
        @returns: A LatexProject object
        """
        vid, pathspec = None, None
        try:
            vid = decode_oxa_link(url)
        except ValueError:
            pathspec = decode_url(url)

        latex_project = cls(session, target_folder, user_options)

        logging.info("Creating folder strcture in %s", {target_folder})
        latex_project.create_folders()

        if vid:
            latex_project.add_article(vid.project, vid.block, vid.version, tex_format)
        else:
            if not pathspec.block:
                raise ValueError("URL does not include a block id")
            latex_project.add_article(
                pathspec.project,
                pathspec.block,
                pathspec.version,
                tex_format,
                tagged_blocks,
            )

        latex_project.reconcile()
        return latex_project

    def create_folders(self):
        logger.info("Creating %s", self._assets_folder)
        os.makedirs(self._assets_folder, exist_ok=True)

        logger.info("Creating %s", self._images_folder)
        os.makedirs(self._images_folder, exist_ok=True)

    def next_index(self):
        return len(self._project_items)

    @log_and_raise_errors(lambda *args: "Could not add article to LaTeX project")
    def add_article(
        self,
        project_id: Union[str, Project],
        article_id: str,
        version: int,
        fmt: BlockFormat,
    ):
        logging.info("adding article using ids/names")
        latex_article = LatexArticle(self._session, project_id, article_id)
        latex_article.fetch(fmt, version)
        latex_article.localize(
            self._session,
            self._assets_folder,
            self._reference_list,
            self._figure_list,
        )
        filename = f"{self.next_index()}_{latex_article._block.name}"
        self._project_items.append(
            ProjectItem(
                path=f"documents/{filename}", article=latex_article, filename=filename
            )
        )
        logging.info("added article")

    def reconcile(self):
        for project_item in self._project_items:
            project_item.article.reconcile()

    def dump(self, allowed_tags: Set[str]) -> Tuple[DocModel, List[str], str]:
        logging.info("Dumping data and content...")
        if len(self._project_items) < 1:
            raise ValueError("Need at least one article")

        # TODO - in book mode (compact == False) we would need to
        # return a list of strings
        content: List[str] = []
        self._tagged_content = TaggedContentCollection()
        for project_item in self._project_items:
            latex_content, tagged = project_item.article.dump(allowed_tags)
            content.append(latex_content)
            self._tagged_content.merge(tagged)

        first = self._project_items[0]

        authors = []
        for author in first.article.authors:
            if author.user:
                user = next(
                    (u for u in first.article._users if u.id == author.user), None
                )
                if user:
                    authors.append(
                        dict(
                            username=user.username,
                            name=user.display_name,
                            bio=user.bio,
                            location=user.location,
                            website=user.website,
                            github=user.github,
                            twitter=user.twitter,
                            affiliation=user.affiliation,
                            orcid=user.orcid,
                            curvenote=f"https://curvenote.com/@{user.username}",
                        )
                    )
            elif author.plain:
                authors.append(dict(name=escape_latex(author.plain)))
            else:
                logging.info("found empty author, skipping...")

        SHORT_TITLE_SIZE = 30
        short_title = (
            first.article.title
            if len(first.article.title) < SHORT_TITLE_SIZE
            else f"{first.article.title[:(SHORT_TITLE_SIZE-3)]}..."
        )

        data = DocModel(
            dict(
                doc=dict(
                    oxalink=first.article.oxalink(self._session.site_url),
                    title=escape_latex(first.article.title),
                    short_title=escape_latex(short_title),
                    description=escape_latex(
                        first.article.description if first.article.description else ""
                    ),
                    authors=authors,
                    date=first.article.date,
                    tags=[escape_latex(tag) for tag in first.article.tags],
                ),
                tagged=self._tagged_content,
                options=self._user_options,
            )
        )

        logging.info("Dumping bibtex...")
        bibtex = ""
        if len(self._reference_list) > 0:
            # de-duplicate the bib entries
            bib_entries = list(set(self._reference_list))
            for reference in bib_entries:
                bibtex += str(reference.content) + "\n"

        return data, content, bibtex

    def write(self, allowed_tags: Set[str] = None):
        """
        Will write 3 files
         - docmodel.yml
         - context.tex
         - main.bib
        """
        allowed_tags = set()
        data, content, bibtex = self.dump(allowed_tags)

        docmodel_filename = os.path.join(self._target_folder, "docmodel.yml")
        with open(docmodel_filename, "w") as file:
            yaml.dump(data, file)
        logging.info("DocModel file created: %s", docmodel_filename)

        content_filename = os.path.join(self._target_folder, "content.tex")
        with open(content_filename, "w") as file:
            for chunk in content:
                file.write(chunk)
                file.write("\n")
        logging.info("Content file created: %s", content_filename)

        bib_filename = os.path.join(self._target_folder, "main.bib")
        with open(bib_filename, "w") as file:
            file.write(bibtex)
        logging.info("Bib file created: %s", bib_filename)

        for tag, content in self._tagged_content:
            with open(f"tagged.{tag}.tex", "w") as file:
                file.write(content)
            logging.info("Tagged content file created: tagged.%s.tex", tag)



import os
import subprocess
import tempfile

import pytest
from curvenote_template import LatexBuilder

from curvenote.client import Session
from curvenote.latex import LatexProject, OnlineTemplateLoader
from curvenote.utils import decode_oxa_link


@pytest.mark.parametrize(
    "override_dir,url",
    [
        (
            None,
            "https://curvenote.com/oxa:nJ1zXZpgPHwAIvQtPCe4/efeOVBOpWerZdEqvVtEL.20",
        ),
        (None, "https://curvenote.com/oxa:nJ1zXZpgPHwAIvQtPCe4/OSifiOWbOKeE6SJTiWOr.1"),
    ],
)
def test_pull_article_builtin_template(override_dir, url):
    """
    Pull a full article using the python code - via url but using the baked in default template
    """

    if "CURVENOTE_TOKEN" in os.environ:
        token = os.environ["CURVENOTE_TOKEN"]
        session = Session(token)

        with tempfile.TemporaryDirectory() as tmp_dir:
            if override_dir is not None:
                tmp_dir = override_dir
            loader = OnlineTemplateLoader(tmp_dir)
            template_options, renderer = loader.initialise_with_builtin_template()

            assert decode_oxa_link(url) is not None
            latex_project = LatexProject.build_single_article_by_url(
                tmp_dir, session, url, template_options.tex_format
            )
            builder = LatexBuilder(template_options, renderer, tmp_dir)
            builder.build(*latex_project.dump(template_options.get_allowed_tags()))


@pytest.fixture(params=["default", "plain_latex"], name="template_name")
def _template_name(request):
    return request.param


def test_pull_article_with_template(template_name):
    """
    Pull a full article using the python code - via url but pulling the basic template from the API
    """

    if "CURVENOTE_TOKEN" in os.environ:
        token = os.environ["CURVENOTE_TOKEN"]
        session = Session(token)

        with tempfile.TemporaryDirectory() as tmp_dir:
            loader = OnlineTemplateLoader(tmp_dir)
            template_options, renderer = loader.initialise_from_template_api(
                session, template_name
            )

            if (
                "CURVENOTE_ENV" in os.environ
                and os.environ["CURVENOTE_ENV"] == "development"
            ):
                url = "https://localhost:8083/oxa:9NOAIcplsbvUjvKy9PuA/p4uJR9W9gfbz3gfIPa96.27"
            else:
                url = "https://curvenote.com/oxa:nJ1zXZpgPHwAIvQtPCe4/efeOVBOpWerZdEqvVtEL.20"
            assert decode_oxa_link(url) is not None
            latex_project = LatexProject.build_single_article_by_url(
                tmp_dir, session, url, template_options.tex_format
            )
            builder = LatexBuilder(template_options, renderer, tmp_dir)
            builder.build(*latex_project.dump(template_options.get_allowed_tags()))


@pytest.fixture(params=[None, "default", "plain_latex"], name="cli_template_name")
def _cli_template_name(request):
    return request.param


def test_cli_build_latex_with_template(cli_template_name):
    """
    Pull a full article using the cli and build it with the template
    """

    if "CURVENOTE_TOKEN" in os.environ:
        token = os.environ["CURVENOTE_TOKEN"]

        with tempfile.TemporaryDirectory() as tmp_dir:
            if (
                "CURVENOTE_ENV" in os.environ
                and os.environ["CURVENOTE_ENV"] == "development"
            ):
                url = "https://localhost:8083/oxa:9NOAIcplsbvUjvKy9PuA/p4uJR9W9gfbz3gfIPa96.27"
            else:
                url = "https://curvenote.com/oxa:nJ1zXZpgPHwAIvQtPCe4/efeOVBOpWerZdEqvVtEL.17"
            assert decode_oxa_link(url) is not None

            CLI_CMD = f"python -m curvenote build-latex {tmp_dir} --url={url}"
            if cli_template_name is not None:
                CLI_CMD = CLI_CMD + f" --template={cli_template_name}"
            ret_val = subprocess.run(CLI_CMD, shell=True)
            assert ret_val.returncode == 0


def test_cli_pull_as_latex():
    """
    Pull a full article using the cli
    """

    if "CURVENOTE_TOKEN" in os.environ:
        token = os.environ["CURVENOTE_TOKEN"]

        with tempfile.TemporaryDirectory() as tmp_dir:
            if (
                "CURVENOTE_ENV" in os.environ
                and os.environ["CURVENOTE_ENV"] == "development"
            ):
                url = "https://localhost:8083/oxa:9NOAIcplsbvUjvKy9PuA/p4uJR9W9gfbz3gfIPa96.27"
            else:
                url = "https://curvenote.com/oxa:nJ1zXZpgPHwAIvQtPCe4/efeOVBOpWerZdEqvVtEL.17"
            assert decode_oxa_link(url) is not None

            CLI_CMD = f"python -m curvenote pull-as-latex {tmp_dir} --url={url}"
            ret_val = subprocess.run(CLI_CMD, shell=True)
            assert ret_val.returncode == 0



import logging
import re
from typing import Dict, List, NewType, Optional, Set, Tuple, Union

from ..client import Session
from ..models import Block, BlockFormat, BlockKind, BlockVersion, Project, User
from .LatexBlockVersion import LatexBlockVersion
from .TaggedContentCollection import TaggedContentCollection
from .utils import LocalMarker, minimise_oxa_path
from .utils.regex import REF_COMMAND_OXA_REGEX


class LatexArticle:
    """
    Class to represent an article in the latex project.
    With the abilty to fetch, localize its assets and write itself to file.
    """

    def __init__(
        self, session: Session, project_id: Union[str, Project], article_id: str
    ):
        self._session: Session = session
        self._project_id: Union[str, Project] = project_id
        self._article_id: str = article_id
        self._block: Optional[Block] = None
        self._version: Optional[BlockVersion] = None
        self._child_blocks: List[Block] = []
        self._child_versions: List[BlockVersion] = []
        self._latex_block_versions: List[LatexBlockVersion] = []
        self._users: List[User] = []

        logging.info("created article")

    @property
    def title(self):
        if self._block:
            return self._block.title
        return ""

    @property
    def description(self):
        if self._block:
            return self._block.description
        return None

    @property
    def authors(self):
        if self._block is None:
            return []
        return self._block.authors if self._block.authors is not None else []

    @property
    def author_names(self):
        names = []
        for author in self.authors:
            if author.user:
                for user in self._users:
                    if user.id == author.user:
                        names.append(user.display_name)
            else:
                names.append(author.plain)
        return names

    @property
    def date(self):
        if self._version:
            return self._version.date or self._version.date_created
        return None

    @property
    def tags(self) -> List[str]:
        if self._block is None:
            return []
        return self._block.tags if self._block.tags is not None else []

    def oxalink(self, base: str):
        if self._version is None:
            return None
        v = self._version
        return f"{base}/oxa:{v.id.project}/{v.id.block}.{v.id.version}"

    def fetch(self, fmt: BlockFormat, version: Optional[int] = None):
        """Download article and all children in Latex format

        Raises ValueError if download fails.
        """
        logging.info("article.fetch")
        block = self._session.get_block(
            self._project_id, self._article_id, kind=BlockKind.article
        )
        logging.info("article.fetch got block")
        version_to_fetch = version or block.latest_version
        children = self._session.get_version_with_children(block, version_to_fetch, fmt)
        if len(children.errors) > 0:
            logging.error("There were errors fetching some children")
            for error in children.errors:
                logging.error(error)

        if children.versions.items[0].kind != BlockKind.article:
            raise ValueError("Expected first child to be an article")
        self._block, *self._child_blocks = children.blocks.items
        self._version, *self._child_versions = children.versions.items
        logging.info(
            f"Processing Article: {self._version.id.project}/"
            f"{self._version.id.block}/versions/{self._version.id.version}"
        )
        logging.info("fetch complete")

    def localize(
        self,
        session: Session,
        assets_folder: str,
        reference_list: List[LocalMarker],
        figure_list: List[LocalMarker],
    ):
        """
            Parse article content, pull assets to local storage and make usable local
            references/labels available to commands as needed

        - images
        - authors
        - citations
        - href
        """
        self._localize_authors(session)
        self._localize_content(session, assets_folder, reference_list, figure_list)

    def reconcile(self):
        """
        For each register figure replace any references found in the content with
        """
        if len(self._latex_block_versions) == 0:
            logging.error("Trying to reconcile article before it was localized")
            return

        for latex in self._latex_block_versions:
            latex.reconcile()

    def _localize_authors(self, session: Session):
        for author in self.authors:
            if author.user is not None:
                try:
                    self._users.append(session.get_user(author.user))
                except ValueError as err:
                    logging.info("Could not get user %s: %s", author.user, err)
                    continue
        logging.info("Localized authors")

    def _localize_content(
        self,
        session: Session,
        assets_folder: str,
        reference_list: List[LocalMarker],
        figure_list: List[LocalMarker],
    ):
        """
        Ignores blocks of any type other than Content and Output.

        @param session: Session to use to get assets
        @param assets_folder: Folder to store assets
        @param reference_list: List of local references
        @param figure_list: List of local figures
        @param tagged_blocks: Set of block tags that should be used to filter special content
        """
        if len(self._child_blocks) == 0:
            logging.warning("Article has no child blocks")
        if len(self._child_versions) == 0:
            logging.warning("Article has no child versions")
        for block, version in zip(self._child_blocks, self._child_versions):
            # pylint: disable=broad-except
            latex = LatexBlockVersion(
                session, assets_folder, reference_list, figure_list, block, version
            )
            try:
                # localizing the block first, even though we may not use the content
                latex.localize()
                self._latex_block_versions.append(latex)
            except Exception as err:
                logging.error("Error localizing block %s: %s", block, err)
                continue
        logging.info("Localized content and references")

    def dump(self, allowed_tags: Set[str]) -> Tuple[str, TaggedContentCollection]:
        """
        Dump article to a string, and tagged content collection

        @param tagged_blocks: Set of block tags that should be used to filter special content
        @return: Tuple of (article content, TaggedContentCollection)
        """
        logging.info("Dumping article content")

        content = ""
        tagged_content = TaggedContentCollection()
        content_flow_idx = 0
        for latex_block in self._latex_block_versions:
            # pylint: disable=broad-except
            try:
                # handle tagged content
                if "no-export" in set(latex_block.tags):
                    logging.info("Found no-export tag, skipping block")
                    continue

                allowed_and_present_tags = sorted(
                    set(allowed_tags) & set(latex_block.tags)
                )
                if len(allowed_and_present_tags) > 0:
                    logging.info("Found tagged content in article")
                    for tag in allowed_and_present_tags:
                        logging.info("Found tag %s", tag)
                        tagged_content.add(tag, latex_block.content)
                    # tagged blocks are removed from normal content flow
                    continue

                # continue on to normal content flow
                if content_flow_idx == 0:
                    content += f"{latex_block.content}\n"
                else:
                    content += f"\n{latex_block.content}\n"
                content_flow_idx += 1
            except Exception as err:
                logging.error("Error dumping block %s: %s", latex_block, err)
                continue

        return content, tagged_content
